from aksharamukha.transliterate import convert
import io
import os
from lxml import etree
from zipfile import ZipFile
from bs4 import BeautifulSoup
import warnings

MS_WORD_TEXT_SCHEMA = "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}t"

def process(src, tgt, file, nativize=True, pre_options=[], post_options=[], is_api_mode=False):
    if os.path.isfile(file):
        if '.txt' in file:
            convert_txt(src, tgt, file, nativize, pre_options, post_options, is_api_mode)
        elif '.docx' in file:
            convert_docx(src, tgt, file, nativize, pre_options, post_options, is_api_mode)
        elif '.htm' in file:
            convert_html(src, tgt, file, nativize, pre_options, post_options, is_api_mode)
        else:
            warnings.warn('Only .txt, .html, .docx formats are supported for file conversion. Input file not processed.')
    else:
        warnings.warn('Not a file. Please pass a file as an argument')

def convert_txt(src, tgt, file, nativize=True, pre_options=[], post_options=[], is_api_mode=False):
    with open(file, 'r', encoding='utf8') as f:
        txt = f.read()

    file_parts = os.path.splitext(file)
    new_file = file_parts[0] + '_' + src + tgt + file_parts[1]

    with open(new_file, 'w', encoding='utf8') as f:
        f.write(convert(src, tgt, txt, nativize, pre_options, post_options))

    print('The following file has been created: ' + new_file)

def convert_html(src, tgt, file, nativize=True, pre_options=[], post_options=[], is_api_mode=False):
    if is_api_mode:
        html = file
    else:
        with open(file, 'r', encoding='utf8') as f:
            html = f.read()

    soup = BeautifulSoup(html)

    for node in soup.findAll(text=True):
        new_node = node.replace(node.string, convert(src, tgt, node.string, nativize, pre_options, post_options))
        node.replace_with(new_node)

    if is_api_mode:
        return soup.prettify()
    else:
        file_parts = os.path.splitext(file)
        new_file = file_parts[0] + '_' + src + tgt + file_parts[1]

        with open(new_file, 'w', encoding='utf8') as f:
            f.write(soup.prettify())

        print('The following file has been created: ' + new_file)

def convert_docx(src, tgt, file, nativize=True, pre_options=[], post_options=[], is_api_mode=False):
    conversion_data = {
        'source': src,
        'target': tgt,
        'nativize': nativize,
        'pre_options': pre_options,
        'post_options': post_options
    }

    new_zip = io.BytesIO()

    if is_api_mode:
        import re
        from io import BytesIO
        from base64 import b64decode
        file = BytesIO(b64decode(file))

    with ZipFile(file, 'r') as old_archive:
        with ZipFile(new_zip, 'w') as new_archive:
            parse_content(conversion_data, old_archive, new_archive)

    file = flush_zip_to_file(file, new_zip, is_api_mode, src, tgt)

    return file


def parse_content(data, old_archive, new_archive):
    for item in old_archive.filelist:
        if item.filename.startswith('word/') and item.filename.endswith('.xml'):
            xml_file_to_convert(data, old_archive, new_archive, item)
        else:
            new_archive.writestr(item, old_archive.read(item.filename))


def xml_file_to_convert(data, old_archive, new_archive, item):
    with old_archive.open(item, 'r') as xml_file:
        tree = etree.parse(xml_file)
        for element in tree.getroot().iter():
            if element.tag == MS_WORD_TEXT_SCHEMA:
                element.text = convert(data['source'],
                                       data['target'],
                                       element.text,
                                       data['nativize'],
                                       data['post_options'],
                                       data['pre_options'])

        string = etree.tostring(tree.getroot(),
                                xml_declaration=True,
                                encoding="UTF-8",
                                standalone=True)

        new_archive.writestr(item, string)


def flush_zip_to_file(file, zip, is_api_mode, src, tgt):
    if is_api_mode:
        # in-memory stream
        file = io.BytesIO(zip.getvalue())
        return file
    else:
        # path
        file_parts = os.path.splitext(file)
        new_file = file_parts[0] + '_' + src + '_' + tgt + file_parts[1]

        with open(new_file, 'wb') as f:
            f.write(zip.getvalue())

        print('The following file has been created: ' + new_file)



# -*- coding: utf-8 -*-

# Script Mapping for Devanagari

VowelMap =  [
            '\u0B85',
            '\u0B86',
            '\u0B87',
            '\u0B88',
            '\u0B89',
            '\u0B8A',
            '\u0BB0\u0BC1\u02BD',
            '\u0BB0\u0BC2\u02BD',
            '\u0BB2\u0BC1\u02BD',
            '\u0BB2\u0BC2\u02BD',
            '\u0B8F',
            '\u0B90',
            '\u0B93',
            '\u0B94',
            ]

SouthVowelMap = [
                '\u0B8E',
                '\u0B92',
                ]

ModernVowelMap = [
                 '\u0B8E\u02BD',
                 '\u0B86\u02BD',
                 ]

SinhalaVowelMap = [
                  '\u0B8F\u02BD'
                  ]

VowelSignMap =  [
                '\u0BBE',
                '\u0BBF',
                '\u0BC0',
                '\u0BC1',
                '\u0BC2',
                '\u0BCD\u0BB0\u0BC1\u02BD',
                '\u0BCD\u0BB0\u0BC2\u02BD',
                '\u0BCD\u0BB2\u0BC1\u02BD',
                '\u0BCD\u0BB2\u0BC2\u02BD',
                '\u0BC7',
                '\u0BC8',
                '\u0BCB',
                '\u0BCC',
                ]

SouthVowelSignMap = [
                    '\u0BC6',
                    '\u0BCA',
                    ]

ModernVowelSignMap =[
                    '\u0BC6\u02BD',
                    '\u0BBE\u02BD']

SinhalaVowelSignMap = [
                      '\u0BC7\u02BD'
                      ]

AyogavahaMap = [
               '\u0BAE\u0BCD\u02BD',
               '\u0BAE\u0BCD\u02BD',
               '꞉' # Modifying letter colon for Visarga
               ]

ViramaMap =  [
             '\u0BCD'
             ]

ConsonantMap =  [
                '\u0B95',
                '\u0B95\u02BD',
                '\u0B95\u02BD',
                '\u0B95\u02BD',
                '\u0B99',

                '\u0B9A',
                '\u0B9A\u02BD',
                '\u0B9A\u02BD',
                '\u0B9A\u02BD',
                '\u0B9E',

                '\u0B9F',
                '\u0B9F\u02BD',
                '\u0B9F\u02BD',
                '\u0B9F\u02BD',
                '\u0BA3',

                '\u0BA4',
                '\u0BA4\u02BD',
                '\u0BA4\u02BD',
                '\u0BA4\u02BD',
                '\u0BA8',

                '\u0BAA',
                '\u0BAA\u02BD',
                '\u0BAA\u02BD',
                '\u0BAA\u02BD',
                '\u0BAE',

                '\u0BAF',
                '\u0BB0',
                '\u0BB2',
                '\u0BB5',

                '\u0B9A\u02BD',
                '\u0B9A\u02BD',
                '\u0B9A\u02BD',
                '\u0B95\u02BD'
                ]

SouthConsonantMap = [
                    '\u0BB3',
                    '\u0BB4',
                    '\u0BB1',
                    '\u0BA9'
                    ]

NuktaConsonantMap =  [
                     '\u02BD\u0B95',
                     '\u02BD\u0B95',
                     '\u02BD\u0B95',
                     '\u02BD\u0B9A',
                     '\u02BD\u0B9F',
                     '\u02BD\u0B9F',
                     '\u02BD\u0BAA',
                     '\u02BD\u0BAF'
                     ]

SinhalaConsonantMap =[
                     '\u0B99\u0BCD\u02BD\u0B95',
                     '\u0B9E\u0BCD\u02BD\u0B9A',
                     '\u0BA3\u0BCD\u02BD\u0B9F',
                     '\u0BA8\u0BCD\u02BD\u0BA4',
                     '\u0BAE\u0BCD\u02BD\u0BAA',
                      ]

NuktaMap = [
           '\u02BD\u02BD'
           ]

OmMap = [
        'ஓம்'
        ]

SignMap =[
         '\u02BD\u02BD',
         '.',
         '..'
         ]

Aytham =['\u0B83']

NumeralMap = [
             '0',
             '1',
             '2',
             '3',
             '4',
             '5',
             '6',
             '7',
             '8',
             '9'
            ]

from ... import GeneralMap as GM

import os
GM.add_additional_chars(dict([(charlist, globals()[charlist]) for charlist in GM.CharmapLists]),
                        os.path.splitext(__file__)[0].split(os.path.sep)[-1])



from aksharamukha import transliterate, GeneralMap

target_scripts = GeneralMap.IndicScripts + GeneralMap.Roman + GeneralMap.SemiticScripts

source_scripts = ['Devanagari', 'IAST']
source_texts = ['heart', 'lalitavistara', 'short']

for source_script in source_scripts:
    for target_script in target_scripts:
        for text in source_texts:
            with open('test_texts/test_' + source_script + '_' + target_script +  '_' + text + '.txt', 'w') as fo:
                with open('test_texts/source_' + source_script + '_' + text + '.txt', 'r') as fi:
                    fo.write(transliterate.process(source_script, target_script, fi.read()))



# -*- coding: utf-8 -*-

## Include AnusvaraToNasal for Santali, TolongSikhi, Limbu, Lepcha

## Generate without Short Vowels - Only include for preserver "Source"

from . import PostProcess as PP
from . import GeneralMap as GM

def ApplyScriptDefaults(Strng,Source,Target, PostOptions=[]):
    Options = []

    #print('Target is ' + Target)

    if Target in GM.IndicScripts:
        Options += []#['RemoveDiacritics'] #Sometimes you may wanna keep it. Adjust options

    if Target == 'Telugu':
        Options += ['NasalToAnusvara','MToAnusvara', 'TeluguRemoveNukta', 'TeluguRemoveAeAo']

    elif Target == 'Kannada':
        Options += ['NasalToAnusvara','MToAnusvara']

    elif Target == 'Sinhala':
        Options += ['SinhalaChandrbinduAnusvara']

    elif Target == 'Khojki':
        Options += ['KhojkiRRI']

    elif Target == 'Nandinagari':
        Options += ['NasalToAnusvara']

    elif Target == 'Hebrew':
        Options += ['HebrewVetVav', 'HebrewnonFinalShort']

    elif Target == 'Malayalam':
        Options += ['MalayalamAnusvaraNasal', 'MToAnusvara', 'MalayalamremoveHistorical']

    elif Target == 'Tamil':
        Options += ['TamilNaToNNa','AnusvaraToNasal', 'TamilpredictDentaNa']

    elif Target == 'Bengali':
        if 'BengaliRaBa' in PostOptions:
            Options += ['YaToYYa','AnusvaraToNasal', 'BengaliConjunctVB']
        else:
            Options += ['AnusvaraToNasal', 'VaToBa','YaToYYa', 'BengaliConjunctVB']

    elif Target == 'MeeteiMayek':
        Options += ['MeeteiMayekremoveHistorical']

    elif Target == 'Limbu':
        Options += ['LimburemoveHistorical']

    elif Target == 'Assamese':
        Options += ['YaToYYa','AnusvaraToNasal']

    elif Target == 'Oriya':
        Options += ['AnusvaraToNasal', 'OriyaVa','YaToYYa']

    elif Target == 'Chakma':
        Options += ['YaToYYa']

    elif Target == 'Gurmukhi':
        Options += ['GurmukhiCandrabindu', 'GurmukhiTippiBindu','GurmukhiTippiGemination']

    elif Target == 'Saurashtra':
        Options += ['SaurashtraHaru']

    elif Target == 'Ahom':
        Options +=['AhomClosed']

    elif Target == 'Tibetan':
        Options += ['TibetanRemoveVirama', 'TibetanRemoveBa']

    elif Target == "Thaana":
        Options += ['ThaanaRemoveHistorical']

    elif Target == "Avestan":
        Options += ['AvestanConventions']

    elif Target == "Sundanese":
        Options += ['SundaneseRemoveHistoric']

    elif Target == "Multani":
        Options += ['MultaniAbjad']

    elif Target == "Modi":
        Options += ['ModiRemoveLong']

    elif Target == "WarangCiti":
        Options += ['WarangCitiModernOrthogaphy']

    elif Target == "Latn":
        ## Also make changes in FixSemiticPreProcess
        if Source == 'Arab':
            Options += ['arabizeLatn']
        elif Source == 'Arab-Ur' or Source == 'Arab-Pa' or Source == 'Arab-Fa':
            Options += ['urduizeLatn']
        elif Source == 'Syrn':
            Options += ['syricizeLatn']
        elif Source == 'Syrj' or Source == 'Hebr':
            Options += ['hebraizeLatn']

    elif Target == "Arab":
        Options += ['ArabRemoveAdditions']

    else:
        Options += []

    for Option in Options:
        if Option.find(Target) != -1:
            Strng = getattr(PP,Option)(Strng)
        else:
            Strng = getattr(PP,Option)(Strng,Target)

    return Strng


from aksharamukha import transliterate

print(transliterate.process('hk', 'siddham', 'buddhaH'))

print(transliterate.process('autodetect', 'iast', 'ꯃꯤꯇꯩ_ꯃꯌꯦꯛ'))

print(transliterate.process('HK', 'Tamil', 'maMgaLa', False))

print(transliterate.process('HK', 'Tamil', 'bRhaspati gaMgA', False, post_options = ['TamilSubScript','TamilRemoveApostrophe']))

print(transliterate.process('Thai', 'Devanagari', 'พุทธัง สะระณัง คัจฉามิ', pre_options=['ThaiOrthography']))

print(transliterate.process('autodetect', 'IAST', 'พุทธัง สะระณัง คัจฉามิ'))

print(transliterate.process('autodetect', 'Vatteluttu', 'พุทธัง สะระณัง คัจฉามิ'))

print(transliterate.auto_detect('ꯃꯤꯇꯩ_ꯃꯌꯦꯛ'))

print(transliterate.process('Devanagari', 'IAST', 'धर्म भारत की श्रमण परम्परा से निकला धर्म और दर्शन है', pre_options=['RemoveSchwaHindi']))

print(transliterate.process('deva', 'taml', 'धर्म भारत की ', param="script_code"))

print(transliterate.process('hi', 'ur', 'धर्म भारत की ', param="lang_code"))

print(transliterate.process('sanskrit', 'telugu', 'धर्म भारत की ', param="lang_name"))

print(transliterate.process('hindi', 'kannada', 'धर्म भारत की ', param="lang_name"))

print(transliterate.process('devanagari', 'granthapandya', 'धर्म'))

print(transliterate.process('hi', 'pa', 'धर्म भारत की ', param="lang_code"))
print(transliterate.process('deva', 'arab', 'धर्म भारत की ', param="script_code"))
print(transliterate.process('autodetect', 'latn-iast', 'धर्म भारत की ', param="script_code"))
print(transliterate.process('la-hK', 'pa-guru', 'namo buddhAya', param="lang_code"))

print(transliterate.process('hi-Deva', 'hi-kthi', 'धर्म भारत की ', param="lang_code"))
print(transliterate.process('hi-Deva', 'mak', 'धर्म भारत की ', param="lang_code"))
print(transliterate.process('hi-Deva', 'cyrl', 'धर्म भारत की ', param="script_code"))
print(transliterate.process('sa-Deva', 'ru', 'धर्म भारत की ', param="lang_code"))



## multiple orthographies associated with script example




from aksharamukha import transliterate

import unittest

class TestStringMethods(unittest.TestCase):

    def test_a(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'အ'), '’a')

    def test_au(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'ဩ ဪ ကော ကော်'), 'o oʻ ko koʻ')

    def test_anusvara(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'ကံ ကာံ အံ အိုံ'), 'kaṃ kāṃ ’aṃ ’uiṃ')

    def test_vowels(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'အ အာ အိ အီ အု အူ အေ အဲ အော အော် အို'), \
            '’a ’ā ’i ’ī ’u ’ū ’e ’ai ’o ’oʻ ’ui')

    def test_subjoined(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'မျှ ကြွ လွှ မြွှင်း'), \
            'myha krva lvha mrvhaṅʻʺ')

    def test_virama_final(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'တက္ကသိုလ် တိရစ္ဆာန်'), \
            'takkasuilʻ tiracchānʻ')

    def test_superscript(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'သင်္ဃ ဓရ်္မ'), \
            'saṅgha dharma')

    def test_ui(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'အို တို ကို'), \
            '’ui tui kui')

    def test_tone_1(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'ပို့ ပိုး'), \
            'puiʹ puiʺ')

    def test_tone_2(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'ပင့်'), \
            'paṅʻʹ')

    def test_great_ssa_nna(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'တဿ ပညာ'), \
            'tassa paññā')

    def test_abbr(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', '၏ ၍ ၌ ၎'), \
            'e* r* n* l*')

    def test_punct(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', '၊ ။'), \
            ', .')

    def test_ll(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'ပါဠိ'), \
            'pāḷi')

    def test_example_1(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'ကော်မီတီ ဥပုသ် ပန်းကန် ကော်ပြန့်'), \
            'koʻmītī upusʻ panʻʺkanʻ koʻpranʻʹ')

    def test_example_2(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', 'မောင်မောင်စိုးတင့် စောစိုင်မောင် ချန်ရီစိန် ကဲနက်ဘစိန် မြသီတာ ပဒေသရာဇာ', \
            pre_options=['segmentBurmeseSyllables']), \
            ' moṅʻ moṅʻ cuiʺ taṅʻʹ   co cuiṅʻ moṅʻ   khyanʻ rī cinʻ   kai nakʻ bha cinʻ   mra sī tā   pa de sa rā jā')

    def test_example_3(self):
        self.assertEqual(transliterate.process('Burmese', 'RomanLoC', '''မဟာသမိုင်းတော်ကြီးညွန့်ပေါင်း
ယောအတွင်းဝန်ဦးဖိုးလှိုင်
ဒုဋ္ဌဂါမဏိမင်းကြီးဝတ္ထု'''), \
            '''mahāsamuiṅʻʺtoʻkrīʺññvanʻʹpoṅʻʺ
yo’atvaṅʻʺvanʻūʺphuiʺlhuiṅʻ
duṭṭhagāmaṇimaṅʻʺkrīʺvatthu''')

    def test_a_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', '’a'), 'အ')

    def test_au_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese',  'o oʻ ko koʻ'), 'ဩ ဪ ကော ကော်')

    def test_anusvara_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'kaṃ kāṃ ’aṃ ’uiṃ'), 'ကံ ကာံ အံ အိုံ')
        ## wrong anu
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'kaṁ kāṁ ’aṁ ’uiṁ'), 'ကံ ကာံ အံ အိုံ')


    def test_vowels_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', '’a ’ā ’i ’ī ’u ’ū ’e ’ai ’o ’oʻ ’ui'), \
            'အ အာ အိ အီ အု အူ အေ အဲ အော အော် အို')
        # wrong accent
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'ʼa ʼā ʼi ʼī ʼu ʼū ʼe ʼai ʼo ʼoʻ ʼui'), \
            'အ အာ အိ အီ အု အူ အေ အဲ အော အော် အို')

    def test_subjoined_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese',  'myha krva lvha mrvhaṅʻʺ'), \
            'မျှ ကြွ လွှ မြွှင်း')

    def test_virama_final_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese',  'takkasuilʻ tiracchānʻ'), \
           'တက္ကသိုလ် တိရစ္ဆာန်')

    def test_superscript_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'saṅgha dharma'), \
            'သင်္ဃ ဓရ်္မ')

    def test_ui_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', '’ui tui kui'), \
            'အို တို ကို')
        ## wrong accent
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'ʼui tui kui'), \
            'အို တို ကို')

    def test_tone_1_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'puiʹ puiʺ'), \
            'ပို့ ပိုး')
        ## the wrong accents
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'pui´ pui˝'), \
            'ပို့ ပိုး')

    def test_tone_2_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'paṅʻʹ'), \
          'ပင့်')
        ## wrong accents
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'paṅʻ´'), \
          'ပင့်')

    def test_great_ssa_nna_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'tassa paññā'), \
            'တဿ ပညာ')

    def test_abbr_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'e* r* n* l*'), \
            '၏ ၍ ၌ ၎')

    def test_punct_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', ', .'), \
            '၊ ။')

    def test_ll_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'pāḷi'), \
            'ပါဠိ')

    def test_example_1_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'koʻmītī upusʻ panʻʺkanʻ koʻpranʻʹ'), \
            'ကော်မီတီ ဥပုသ် ပန်းကန် ကော်ပြန့်')

    def test_example_2_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', 'moṅʻ moṅʻ cuiʺ taṅʻʹ co cuiṅʻ moṅʻ khyanʻ rī cinʻ kai nakʻ bha cinʻ mra sī tā pa de sa rā jā', \
            post_options=['removeSegmentSpacesBurmese']), \
            'မောင်မောင်စိုးတင့်စောစိုင်မောင်ချန်ရီစိန်ကဲနက်ဘစိန်မြသီတာပဒေသရာဇာ')

    def test_example_3_rev(self):
        self.assertEqual(transliterate.process('RomanLoC', 'Burmese', '''mahāsamuiṅʻʺtoʻkrīʺññvanʻʹpoṅʻʺ
yo’atvaṅʻʺvanʻūʺphuiʺlhuiṅʻ
duṭṭhagāmaṇimaṅʻʺkrīʺvatthu'''), \
            '''မဟာသမိုင်းတော်ကြီးညွန့်ပေါင်း
ယောအတွင်းဝန်ဦးဖိုးလှိုင်
ဒုဋ္ဌဂါမဏိမင်းကြီးဝတ္ထု''')

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-

# Script Mapping for ISO

VowelMap =  [
            'a',
            'aa',
            'i',
            'ee',
            'u',
            'oo',
            'Ru',
            'RU',
            '~lu',
            '~lU',
            'E',
            'ai',
            'O',
            'au'
            ]

SouthVowelMap = [
                'e',
                'o',
                ]

ModernVowelMap = [
                 '~e',
                 '~o',
                 ]

SinhalaVowelMap = [
                  '~~e'
                  ]

VowelSignMap =  VowelMap[1:]

SouthVowelSignMap = SouthVowelMap[:]

ModernVowelSignMap = ModernVowelMap[:]

SinhalaVowelSignMap = SinhalaVowelMap[:]

AyogavahaMap = [
               '~M',
               'M',
               'H'
               ]

ViramaMap =  [
             '\u00D7'
             ]

ConsonantMap =  [
                '\u006B',
                '\u006B\u0068',
                '\u0067',
                '\u0067\u0068',
                '~g',

                'c',
                'C',
                '\u006A',
                '\u006A\u0068',
                '~j',
                'T',
                'Th',
                'D',
                'Dh',
                'N',

                't',
                'th',
                'd',
                'dh',
                '\u006E',

                '\u0070',
                '\u0070\u0068',
                '\u0062',
                '\u0062\u0068',
                '\u006D',

                '\u0079',
                '\u0072',
                '\u006C',
                '\u0076',

                'S',
                'Sh',
                '\u0073',
                '\u0068',
                ]

SouthConsonantMap = [
                    'L',
                    'Lx',
                    'rx',
                    'nx'
                    ]

NuktaConsonantMap =  [
                     'kx',
                     'Kx',
                     'gx',
                     'z',
                     'Dx',
                     'Dhx',
                     'f',
                     'yx'
                     ]

SinhalaConsonantMap =[
                     '~ng',
                     '~nj',
                     '~nD',
                     '~nd',
                     '~mb',
                      ]

NuktaMap = [
           'x'
           ]

OmMap = [
        'oum'
        ]

SignMap =[
         '&',
         '|',
         '||'
         ]

Aytham =['~H']

NumeralMap = [
             '\u0030',
             '\u0031',
             '\u0032',
             '\u0033',
             '\u0034',
             '\u0035',
             '\u0036',
             '\u0037',
             '\u0038',
             '\u0039',
             ]

from ... import GeneralMap as GM

import os
GM.add_additional_chars(dict([(charlist, globals()[charlist]) for charlist in GM.CharmapLists]),
                        os.path.splitext(__file__)[0].split(os.path.sep)[-1])


# -*- coding: utf-8 -*-

# Script Mapping for ISO

VowelMap =  [
            '\u0061',
            'a',
            '\u0069',
            'i',
            '\u0075',
            'u',
            'ri',
            'ri',
            'li',
            'li',
            'e',
            'ai',
            'o',
            'au'
            ]

SouthVowelMap = [
                'e',
                'o',
                ]

ModernVowelMap = [
                 'a',
                 'o',
                 ]

SinhalaVowelMap = [
                  'e'
                  ]

VowelSignMap =  VowelMap[1:]

SouthVowelSignMap = SouthVowelMap[:]

ModernVowelSignMap = ModernVowelMap[:]

SinhalaVowelSignMap = SinhalaVowelMap[:]

AyogavahaMap = [
               '\u004D',
               '\u004D',
               'h'
               ]

ViramaMap =  [
             '\u00D7'
             ]

ConsonantMap =  [
                '\u006B',
                '\u006B\u0068',
                '\u0067',
                '\u0067\u0068',
                'ng',

                'ch',
                'chh',
                '\u006A',
                '\u006A\u0068',
                'nj',

                't\'',
                'th',
                'd\'',
                'dh',
                'n',

                't',
                'th',
                'd',
                'dh',
                '\u006E',

                '\u0070',
                '\u0070\u0068',
                '\u0062',
                '\u0062\u0068',
                '\u006D',

                '\u0079',
                '\u0072',
                '\u006C',
                '\u0076',

                'sh',
                'sh',
                '\u0073',
                '\u0068',
                ]

SouthConsonantMap = [
                    'l',
                    'zh',
                    'r',
                    'n'
                    ]

NuktaConsonantMap =  [
                     'q',
                     'kh',
                     'g',
                     'z',
                     'r\'',
                     'r\'h',
                     'f',
                     'y'
                     ]

SinhalaConsonantMap =[
                     'ng',
                     'nj',
                     'nd',
                     'nd',
                     'mb',
                      ]

NuktaMap = [
           '\u02BD\u02BD'
           ]

OmMap = [
        'Om'
        ]

SignMap =[
         '\'',
         '\u002E',
         '\u002E\u002E'
         ]

Aytham =['g']

NumeralMap = [
             '\u0030',
             '\u0031',
             '\u0032',
             '\u0033',
             '\u0034',
             '\u0035',
             '\u0036',
             '\u0037',
             '\u0038',
             '\u0039',
             ]

from ... import GeneralMap as GM

import os
GM.add_additional_chars(dict([(charlist, globals()[charlist]) for charlist in GM.CharmapLists]),
                        os.path.splitext(__file__)[0].split(os.path.sep)[-1])


# -*- coding: utf-8 -*-

from . import PreProcess as PP

def ApplyPreProcessing(Strng,Source,Target):
    if Target == "PhagsPa":
        Options = ['PhagsPaArrange']
    else:
        Options = []

    #print('Getting preprocesssing ')

    for Option in Options:
        if Option.find(Target) != -1:
            Strng = getattr(PP,Option)(Strng,Source)
            #print (Strng)
        else:
            Strng = getattr(PP,Option)(Strng,Target)
            #print(Strng)

    return Strng



# -*- coding: utf-8 -*-

# Script Mapping for ISO

VowelMap =  [
            '\u0061',
            'aa',
            '\u0069',
            'ii',
            '\u0075',
            'uu',
            'ri',
            'rii',
            'r\u309Ai',
            'r\u309Aii',
            'ee',
            'ai',
            'oo',
            'au'
            ]

SouthVowelMap = [
                'e',
                'o',
                ]

ModernVowelMap = [
                 'a',
                 'o',
                 ]

SinhalaVowelMap = [
                  'e'
                  ]

VowelSignMap =  VowelMap[1:]

SouthVowelSignMap = SouthVowelMap[:]

ModernVowelSignMap = ModernVowelMap[:]

SinhalaVowelSignMap = SinhalaVowelMap[:]

AyogavahaMap = [
               'n',
               'n',
               'H'
               ]

ViramaMap =  [
             '\u00D7'
             ]

ConsonantMap =  [
                '\u006B',
                '\u006B',
                '\u0067',
                '\u0067',
                'k\u309A',

                'c',
                'c',
                '\u006A',
                '\u006A',
                'J',

                't',
                't',
                'd',
                'd',
                'n',

                't',
                't',
                'd',
                'd',
                '\u006E',

                '\u0070',
                '\u0070',
                '\u0062',
                '\u0062',
                '\u006D',

                '\u0079',
                'r',
                'r\u309A',
                'v',

                'sh',
                'sh',
                '\u0073',
                '\u0068',
                ]

SouthConsonantMap = [
                'r\u309A',
                'r\u309A',
                    'r',
                    'n'
                    ]

NuktaConsonantMap =  [
                     'k',
                     'k',
                     'g',
                     'z',
                     'r',
                     'r',
                     'f',
                     'y'
                     ]

SinhalaConsonantMap =[
                     'ng',
                     'nj',
                     'nd',
                     'nd',
                     'nb',
                      ]

NuktaMap = [
           '\u02BD\u02BD'
           ]

OmMap = [
        'oon'
        ]

SignMap =[
         '\'',
         '。',
         '。。'
         ]

Aytham =['gu']

NumeralMap = [
             '\u0030',
             '\u0031',
             '\u0032',
             '\u0033',
             '\u0034',
             '\u0035',
             '\u0036',
             '\u0037',
             '\u0038',
             '\u0039',
             ]

from ... import GeneralMap as GM

import os
GM.add_additional_chars(dict([(charlist, globals()[charlist]) for charlist in GM.CharmapLists]),
                        os.path.splitext(__file__)[0].split(os.path.sep)[-1])


import pytest
from aksharamukha import transliterate, GeneralMap, PostProcess, PreProcess

target_scripts = GeneralMap.IndicScripts + GeneralMap.Roman + GeneralMap.SemiticScripts
source_texts = ['heart', 'lalitavistara']
post_options = (d for d in dir(PostProcess) if '__' not in d)

reversible_scripts = GeneralMap.ReversibleScripts + GeneralMap.RomanReversible

def text_compare_reversible_2(source_script, target_script, source_text):
    try:
        with open('test_texts/source_Devanagari'+ '_' + source_text + '.txt', 'r', encoding= 'utf8') as fi:
            src_txt = fi.read()
            dev_to_src = transliterate.process('Devanagari', source_script, src_txt, nativize=False)
            src_to_tgt = transliterate.process(source_script, target_script, dev_to_src, nativize=False)
            tgt_to_src = transliterate.process(target_script, source_script, src_to_tgt, nativize=False)
            assert tgt_to_src == dev_to_src
    except FileNotFoundError:
        ## Fix this ## fix this
        print(source_script, target_script)


def text_compare_reversible(source_script, target_script, source_text):
    try:
        with open('test_texts/source_' + source_script + '_' + source_text + '.txt', 'r', encoding= 'utf8') as fi:
            src_txt = fi.read()
            src_to_tgt = transliterate.process(source_script, target_script, src_txt, nativize=False)
            tgt_to_src = transliterate.process(target_script, source_script, src_to_tgt, nativize=False)
            assert tgt_to_src == src_txt
    except FileNotFoundError:
        ## Fix this ## fix this
        print(source_script, target_script)


def text_compare(source_script, target_script, source_text):
    try:
        with open('test_texts/test_' + source_script + '_' + target_script +  '_' + source_text + '.txt', 'r', encoding= 'utf8') as fo:
            with open('test_texts/source_' + source_script + '_' + source_text + '.txt', 'r', encoding= 'utf8') as fi:
                assert transliterate.process(source_script, target_script, fi.read()) == fo.read()
    except FileNotFoundError:
        print(source_script, target_script)

class TestOverall:
    @pytest.mark.parametrize("source_script", ['Devanagari', 'IAST'])
    @pytest.mark.parametrize("target_script", target_scripts)
    @pytest.mark.parametrize("source_text", source_texts)
    def test_overall(self, source_script, target_script, source_text):
        text_compare(source_script, target_script, source_text)

class TestOverallReversible:
    @pytest.mark.parametrize("source_script", ['Devanagari', 'IAST'])
    @pytest.mark.parametrize("target_script", reversible_scripts)
    @pytest.mark.parametrize("source_text", source_texts)
    def test_overall_reversible(self, source_script, target_script, source_text):
        if target_script != 'Bengali':
            text_compare_reversible(source_script, target_script, source_text)

class TestOverallReversible2:
    @pytest.mark.parametrize("source_script", reversible_scripts)
    @pytest.mark.parametrize("target_script", reversible_scripts)
    @pytest.mark.parametrize("source_text", source_texts)
    def test_overall_reversible_2(self, source_script, target_script, source_text):
        if target_script != 'Bengali' and source_script != 'Bengali':
            text_compare_reversible_2(source_script, target_script, source_text)

class TestPostOptions:
    @pytest.mark.parametrize("post_options", post_options)
    def test_postoptions(self, post_options):
        import json

        with open('test_texts/test_postoptions.json', 'r', encoding= 'utf8') as f:
            texts = json.load(f)
        try:
            testcases = texts[post_options]

            for testcase in testcases:
                for case in testcase["cases"]:
                    assert transliterate.process(testcase["source"], testcase["target"], case[0], post_options=[post_options]) == case[1]
        except KeyError:
            assert 1 == 1

class TestSanity:
    @pytest.mark.parametrize("target_scripts", target_scripts)
    def test_sanity_default(self, target_scripts):
        import json
        with open('test_texts/test_sanity_default.json', 'r', encoding= 'utf8') as f:
            texts = json.load(f)
        try:
            testcases = texts[target_scripts]

            for testcase in testcases:
                for case in testcase["cases"]:
                    nativize = testcase["nativize"] if "nativize" in testcase else True
                    assert transliterate.process(testcase["source"], testcase["target"], case[0], nativize=nativize) == case[1]

                    if "reverse" in testcase:
                        assert transliterate.process(testcase["target"], testcase["source"], case[1], nativize=nativize) == case[0]
        except KeyError:
            assert 1 == 1

class TestMisc:
    def test_bengali_va_default(self):
        assert transliterate.process('HK', 'Bengali', 'savva sabba samba samva sarva sarba ukva ukba utva utba udveda udbodhana bru bra bya byu vra vya vla') == 'সব্ব সব্ব সম্ব সম্ব সর্ব সর্ব উক্ব উক্ব উত্ব উত্ব উদ্বেদ উদ্বোধন ব্রু ব্র ব্য ব্যু ব্র ব্য ব্ল'

    def test_bengali_va_preserve_source(self):
        assert transliterate.process('HK', 'Bengali', 'savva sabba samba samva sarva sarba ukva ukba utva utba udveda udbodhana bru bra bya byu vra vya vla', nativize=False) == 'সভ়্ভ় সব্ব সম্ব সম্ভ় সর্ভ় সর্ব উক্ব উক্‌ব উত্ব উৎ‌ব উদ্বেদ উদ্‌বোধন ব্রু ব্র ব্য ব্যু ভ়্র ভ়্য ভ়্ল'

    def test_bengali_va_old_ra(self):
        assert transliterate.process('HK', 'Bengali', 'savva sabba samba samva sarva sarba ukva ukba utva utba udveda udbodhana bru bra bya byu vra vya vla', post_options=['BengaliRaBa']) == 'সব্ব সৰ্‌‌ৰ সম্‌ৰ সম্ব সর্ব সৰ্ৰ উক্ব উক্‌ৰ উত্ব উত্‌ৰ উদ্বেদ উদ্‌ৰোধন ৰ‍্রু ৰ‍্র ৰ‍্য ৰ‍্যু ব্র ব্য ব্ল'


from aksharamukha import GeneralMap
from aksharamukha.gimeltra import Transliterator
from . import Convert,PostOptions,PostProcess,PreProcess
from . import ConvertFix
import json
import requests
import html
import itertools
from collections import Counter
import unicodedata
import collections
import yaml
import warnings
import langcodes
import os
from inspect import getmembers, isfunction

#import sys
#sys.stdout = None

def removeA(a):
    if a.count('a') == 1:
        return a.replace('a', '')

def unique_everseen(iterable, key=None):
    "List unique elements, preserving order. Remember all elements ever seen."
    # unique_everseen('AAAABBBCCDAABBB') --> A B C D
    # unique_everseen('ABBCcAD', str.lower) --> A B C D
    seen = set()
    seen_add = seen.add
    if key is None:
        for element in itertools.filterfalse(seen.__contains__, iterable):
            seen_add(element)
            yield element
    else:
        for element in iterable:
            k = key(element)
            if k not in seen:
                seen_add(k)
                yield element

def auto_detect(text, plugin = False):
    scripts = []

    for uchar in text:
        try:
            script_name = unicodedata.name(uchar).split(' ')[0].lower()
            if script_name != 'old':
                scripts.append(script_name)
            else:
                scripts.append(unicodedata.name(uchar).split(' ')[1].lower())
        except ValueError:
            pass
            # print('Script not found')

    counts = Counter(scripts)
    script_percent = []

    for script, count  in counts.items():
        percent = count/len(scripts) * 100
        script_percent.append((percent, script))

    #print(sorted(script_percent))

    if not plugin:
        if len(script_percent) > 0:
            script = sorted(script_percent)[-1][1]
        else:
            script = ''
    else:
        #print('here')
        if len(script_percent) > 0:
            if sorted(script_percent)[-1][1] == 'latin':
                script = sorted(script_percent)[-2][1]
            else:
                script = sorted(script_percent)[-1][1]
        else:
            script = ''

    inputScript = script[0].upper() + script[1:]

    laoPali = ['ຆ', 'ຉ', 'ຌ', 'ຎ', 'ຏ', 'ຐ', 'ຑ', 'ຒ', 'ຓ', 'ຘ', 'ຠ', 'ຨ', 'ຩ', 'ຬ', '຺']

    if inputScript == 'Bengali':
        if 'ৰ' in text or 'ৱ' in text:
            inputScript = 'Assamese'
    elif inputScript == 'Lao':
        if any(char in text for char in laoPali):
            inputScript = 'LaoPali'
    elif inputScript == 'Batak':
        inputScript = 'BatakKaro'
    elif inputScript == 'Myanmar':
        inputScript = 'Burmese'

        mon = ['ၚ', 'ၛ', '္ည', 'ၞ', 'ၟ', 'ၠ', 'ဳ', 'ဨ']
        if any([char in text for char in mon]):
            inputScript = 'Mon'

        countSub = {'Shan': 0, 'TaiLaing': 0, 'KhamtiShan': 0}

        text = text.replace('ႃ', '')

        for uchar in text:
            try:
                char = unicodedata.name(uchar).lower()
            except:
                pass
            if 'shan' in char:
                countSub['Shan'] += 1
            elif 'tai laing' in char:
                countSub['TaiLaing'] += 1
            elif 'khamti' in char:
                countSub['KhamtiShan'] += 1

        import operator
        sorted_x = sorted(countSub.items(), key=operator.itemgetter(1))

        if countSub['Shan'] > 0 or countSub['TaiLaing'] > 0 or countSub['KhamtiShan'] > 0:
            inputScript = sorted_x[-1][0]


        #shan = ['ႃ', '\u1086', '\u1084', 'ၵ', 'ၶ', 'ၷ', 'ꧠ', 'ၸ', 'ꧡ', 'ꩡ', 'ꧢ', 'ၺ', 'ꩦ', 'ꩧ', 'ꩨ', 'ꩩ', 'ꧣ', 'ၻ', 'ꩪ', 'ၼ','ၽ', 'ꧤ', 'ႁ', 'ꩮ', 'ၹ', 'ၾ']
        #if any([char in text for char in shan]):
            #inputScript = 'Shan'

    elif inputScript == 'Meetei':
        inputScript = 'MeeteiMayek'
    elif inputScript == 'Dives':
        inputScript = 'DivesAkuru'
    elif inputScript == 'Persian':
        inputScript = 'OldPersian'
    elif inputScript == 'Phags-pa':
        inputScript = 'PhagsPa'
    elif inputScript == 'Ol':
        inputScript = 'Santali'
    elif inputScript == 'Sora':
        inputScript = 'SoraSompeng'
    elif inputScript == 'Syloti':
        inputScript = 'SylotiNagri'
    elif inputScript == 'Tai':
        inputScript = 'TaiTham'
    elif inputScript == 'Warang':
        inputScript = 'WarangCiti'
    elif inputScript == 'Siddham':
        preOptions = 'siddhamUnicode'
    elif inputScript == 'Cyrillic':
        inputScript = 'RussianCyrillic'
    elif inputScript == 'Zanabazar':
        inputScript = 'ZanabazarSquare'
    elif inputScript == 'Syriac':
        inputScript = 'Syre'
        eastern_dia = 'ܲ ܵ ܝܼ ܘܼ ܸ ܹ ܘܿ'.split(' ')
        if any(char in text for char in eastern_dia):
            inputScript = 'Syrn'
        western_dia = 'ܰ ܺ ܶ ّ ܽ'.split(' ')
        if any(char in text for char in western_dia):
            inputScript = 'Syrj'
    elif inputScript == 'Arabic':
        inputScript = 'Arab'
        persian_char = 'چ گ ژ پ هٔ'.split(' ')
        if any(char in text for char in persian_char):
            inputScript = 'Arab-Fa'
        urdu_char = 'ڈ ٹ ڑ ھ'.split(' ')
        if any(char in text for char in urdu_char):
            inputScript = 'Urdu'
        shahmukh_char = 'ݨ لؕ مھ نھ یھ رھ لھ وھ'.split(" ")
        if any(char in text for char in shahmukh_char):
            inputScript = 'Shahmukhi'
    elif inputScript == 'Latin':
        diacritics = ['ā', 'ī', 'ū', 'ṃ', 'ḥ', 'ś', 'ṣ', 'ṇ', 'ṛ', 'ṝ', 'ḷ', 'ḹ', 'ḻ', 'ṉ', 'ṟ', 'ṭ', 'ḍ', 'ṅ', 'ñ']
        Itrans = ['R^i', 'R^I', 'L^i', 'L^I', '.N', '~N', '~n', 'Ch', 'sh', 'Sh']
        semitic = ['ʾ', 'ʿ', 'š', 'w']
        BurmeseRomanLoC = ['´', '˝', 'ʻ']
        if 'ʰ' in text:
            inputScript = 'Titus'
        elif any(char in text for char in semitic):
            inputScript = 'Latn'
        elif any(char in text for char in BurmeseRomanLoC):
            inputScript = 'BurmeseRomanLoC' ## Change for other LoC schemese
        elif any(char in text for char in diacritics):
            if 'ē' in text or 'ō' in text or 'r̥' in text:
                inputScript = 'ISO'
            else:
                inputScript = 'IAST'
        elif any(char in text for char in Itrans):
            inputScript = 'Itrans'
        else:
            inputScript = 'HK'
    elif inputScript in GeneralMap.IndicScripts or inputScript in GeneralMap.LatinScripts or inputScript in ['Hiragana', 'Katakana']:
        pass
    else:
        from . import gimeltra
        tr = gimeltra.Transliterator()
        inputScript = tr.auto_script(text)

    return inputScript

# Cross check with inded convert() funciton within autodetect
# scripts available here must be added there
def detect_preoptions(text, inputScript):
    preoptions = []
    if inputScript == 'Thai':
        textNew = text.replace('ห์', '')
        if '\u035C' in text or '\u0325' in text or 'งํ' in text or '\u0E47' in text:
            preoptions = ['ThaiPhonetic']
        elif '์' in textNew and ('ะ' in text):
            preoptions = ['ThaiSajjhayawithA']
        elif '์' in textNew:
            preoptions = ['ThaiSajjhayaOrthography']
        elif 'ะ' in text or 'ั' in text:
            preoptions =  ['ThaiOrthography']
    elif inputScript == 'Lao' or inputScript == 'LaoPali':
        textNew = text.replace('ຫ໌', '')
        if '໌' in textNew and ('ະ' in text):
            preoptions = ['LaoSajhayaOrthographywithA']
        elif '໌' in textNew:
            preoptions = ['LaoSajhayaOrthography']
        elif 'ະ' in text or 'ັ' in text:
            preoptions = ['LaoTranscription']
    elif inputScript == 'Urdu':
            preoptions = ['UrduShortNotShown']

    return preoptions

def convert(src, tgt, txt, nativize, preoptions, postoptions):
    tgtOld = ""

    ## ALA-LC Romanizations
    ## process as follows, if source is associated with LoC script
    if tgt == 'RomanLoC' and src in GeneralMap.LoCScripts:
    ### swtiches for Source scripts
        if src in GeneralMap.LoCSrcMap:
            src = src + 'LoC'
        elif 'Tham' in src:
            src = 'ThamLoC'

        if src in GeneralMap.LoCTgtMap:
            tgt = src + tgt
        elif src in GeneralMap.LoCTgtISO:
            tgt = 'ISO'

        # the below order for preoptions is important
        #special for east-asian scripts
        if src in GeneralMap.LocPostPre:
            preoptions = preoptions + [tgt + 'Target']
            postoptions =  [tgt + 'Target'] + postoptions
            nativize = False

            if src in ['KhmerLoC']:
                preoptions =  preoptions + ['SchwaFinalKhmerLoC']
        #read from GeneralMap
        else:
            if src in GeneralMap.LoCTgtPostOptions.keys():
                postoptions =  GeneralMap.LoCTgtPostOptions[src] + postoptions
            if src in GeneralMap.LoCTgtPreOptions.keys():
                preoptions =  preoptions + GeneralMap.LoCTgtPreOptions[src]

    ## if source is not associated with LoC script
    if tgt == 'RomanLoC' and src not in GeneralMap.LoCScripts:
        if src in GeneralMap.SemiticScripts:
            tgt = 'ISO233'
        else:
            tgt = 'ISO'

    ## Switches for Target scripts
    ## if target is associated with the loc Script
    if src == 'RomanLoC' and tgt in GeneralMap.LoCScripts:
        if tgt in GeneralMap.LoCSrcMap:
            tgt = tgt + 'LoC'
        elif 'Tham' in tgt:
            tgt = 'ThamLoC'

        if tgt in GeneralMap.LoCTgtMap:
            src = tgt + src
        elif tgt in GeneralMap.LoCTgtISO:
            src = 'ISO'

        if tgt in GeneralMap.LocPostPre:
            preoptions = [src + 'Source'] + preoptions
            postoptions =  [src + 'Source'] + postoptions
            nativize = False
        #read from GeneralMap
        else:
            if tgt in GeneralMap.LoCSrcPostOptions.keys():
                postoptions =  GeneralMap.LoCSrcPostOptions[tgt] + postoptions
            if tgt in GeneralMap.LoCSrcPreOptions.keys():
                preoptions =  preoptions + GeneralMap.LoCSrcPreOptions[tgt]

    ## if target is not associated with the loc Script
    if src == 'RomanLoC' and tgt not in GeneralMap.LoCScripts:
        if tgt in GeneralMap.SemiticScripts:
            src = 'ISO233'
        else:
            src = 'ISO'

    ## LoC Romanizattion ends here
    ## else convert the Loc input into the associated script and then convert it into Loc
    ## Text (Loc) -> Burmese -> non-Burmese
    #if src == 'BurmeseRomanLoC' and tgt != 'Burmese':
    #    txt = convert(src, "Burmese", txt, nativize, preoptions, postoptions)
    #    src = 'Burmese'

    ## Semitic ISO Standards
    ### same as below use intermediates to convert non-associated input/output
    if tgt in GeneralMap.semiticISO.keys():
        if GeneralMap.semiticISO[tgt] != src:
            txt = convert(src, GeneralMap.semiticISO[tgt], txt, nativize, preoptions, postoptions)
            src = GeneralMap.semiticISO[tgt]

        if GeneralMap.semiticISO[tgt] == src:
            preoptions = [tgt + 'Target'] + preoptions
            postoptions =  [tgt + 'Target'] + postoptions
            nativize = False
            tgt = 'Latn'

    if src in GeneralMap.semiticISO.keys():
        if GeneralMap.semiticISO[src] == tgt:
            #print('here1')
            preoptions = [src + 'Source'] + preoptions
            postoptions = [src + 'Source'] + postoptions
            src = 'Latn'
        else:
            #print('here2')
            txt = convert(src, GeneralMap.semiticISO[src], txt, nativize, preoptions, postoptions)
            src = GeneralMap.semiticISO[src]

    '''if src == "Itrans" and '##' in txt:
        textNew = [[i,word] for i, word in enumerate(txt.split("##")) if (i%2 == 0)]
        textRest = [(i,word) for i, word in enumerate(txt.split("##")) if (i%2 == 1)]

        txt = json.dumps(textNew).replace("\\n", "\n")'''

    if tgt == "" or tgt == "Ignore":
        return txt
    if preoptions == [] and postoptions == [] and nativize == False and src == tgt:
        return txt

    ## Semitic Switches - Add Shahmukhi/Sindhi later
    IndicSemiticMapping = { 'Hebrew': 'Hebr', 'Thaana': 'Thaa', 'Urdu': 'Arab-Ur', 'Shahmukhi': 'Arab-Pa'}

    if (tgt in GeneralMap.SemiticScripts or tgt in GeneralMap.semiticISO.keys()) and src in IndicSemiticMapping.keys():
        src = IndicSemiticMapping[src]
    if (src in GeneralMap.SemiticScripts or src in GeneralMap.semiticISO.keys()) and tgt in IndicSemiticMapping.keys():
        tgt = IndicSemiticMapping[tgt]
    if src in IndicSemiticMapping.keys() and tgt in IndicSemiticMapping.keys():
        src = IndicSemiticMapping[src]
        tgt = IndicSemiticMapping[tgt]

    # preserve semitic pattern
    if not nativize and src == 'Hebrew':
        src = 'Hebr'
    if not nativize and src == 'Urdu':
        src = 'Arab-Ur'
    if not nativize and src == 'Shahmukhi':
        src = 'Arab-Pa'
    if not nativize and src == 'Thaana':
        src = 'Thaa'

    if src in ['Arab-Ur', 'Arab-Pa'] and tgt in GeneralMap.IndicScripts:
        txt += '\u05CD'

    ## Semitic to Urdu this is not needed
    #if src in GeneralMap.SemiticScripts and src not in ['Arab-Fa', 'Arab', 'Latn'] and (tgt in ['Urdu', 'Shahmukhi']):
    #    tgt = 'Arab-Fa'

    ## recheck this the logic is missing :: This is not required
    """if src in ['Urdu', 'Shahmukhi'] and tgt == 'Arab':
        #print('Semiticizing Urdu')
        txt = PreProcess.semiticizeUrdu(txt)
        txt = PreProcess.ArabizePersian(txt)
        src = "Arab-Fa"
        tgt = "Arab"

    if src in ['Urdu', 'Shahmukhi'] and tgt == 'Arab-Fa':
        txt = PreProcess.semiticizeUrdu(txt)
        src = "none"
        tgt = "none"

    if src == 'Arab-Fa' and tgt == 'Arab':
        txt = PreProcess.ArabizePersian(txt)
        #src = "none"
        #tgt = "none"

    if src  == 'Arab' and tgt == 'Arab-Fa':
        txt = PreProcess.perisanizeArab(txt)
        src = "none"
        tgt = "none"

    if src  == 'Arab' and tgt in ['Urdu', 'Shahmukhi']:
        txt = PreProcess.perisanizeArab(txt)
        src = "none"
        tgt = "Arab-Fa"

    if src in 'Arab-Fa' and tgt in ['Urdu', 'Shahmukhi']:
        src = "none"
        tgt = "none" """

    ## Semitic Naivization : remove pesky Nuktas
    if nativize:
        if src in GeneralMap.SemiticScripts and tgt in GeneralMap.IndicScripts:
            txt += "\u05CC"

    if src == tgt and (src != 'Hiragana' and src != 'Katakana') and src not in GeneralMap.SemiticScripts:
        tgtOld = tgt
        tgt = "Devanagari"

    ## this has to be before preprocess.preprocess
    if 'indicDandas' in postoptions:
        if src in GeneralMap.pipeScripts and '|' in txt:
            txt = PreProcess.RetainPipeDanda(txt, src)
        elif GeneralMap.CrunchList('SignMap', src)[1] == '।':
            preoptions = preoptions + ['RetainDevangariDanda']
        else:
            if tgt not in GeneralMap.Transliteration:
                postoptions = postoptions + ['Dot2Dandas']
            else:
                postoptions = postoptions + ['Dot2Pipes']

    txt = PreProcess.PreProcess(txt,src,tgt,postoptions,preoptions)

    if 'siddhammukta' in postoptions and tgt == 'Siddham':
        tgt = 'SiddhamDevanagari'
    if 'siddhamap' in postoptions and tgt == 'Siddham':
        tgt = 'SiddhamDevanagari'
    if 'siddhammukta' in preoptions and src == 'Siddham':
        src = 'SiddhamDevanagari'
    if 'LaoNative' in postoptions and tgt == 'Lao':
        tgt = 'Lao2'
    if 'egrantamil' in preoptions and src == 'Grantha':
        src = 'GranthaGrantamil'
    if 'egrantamil' in postoptions and tgt == 'Grantha':
        tgt = 'GranthaGrantamil'
    if 'nepaldevafont' in postoptions and tgt == 'Newa':
        tgt = 'Devanagari'
    if 'ranjanalantsa' in postoptions and tgt == 'Ranjana':
        tgt = 'Tibetan'
        nativize = False
    if 'ranjanawartu' in postoptions and tgt == 'Ranjana':
        tgt = 'Tibetan'
        nativize = False
    if 'BengaliRaBa' in postoptions and tgt == 'Bengali':
        tgt = 'BengaliRaBa'
        nativize = False
    if 'SoyomboFinals' in postoptions and tgt == 'Soyombo':
        txt = '\u02BE' + txt
    if  'BalineseSimplified' in postoptions and src == 'Balinese':
        tgt = 'BalineseSimpleRomanLoC'
    if  'JavaneseSimplified' in postoptions and src == 'Javanese':
        tgt = 'JavaneseSimpleRomanLoC'

    #if src not in GeneralMap.SemiticScripts and tgt == 'Arab' and not nativize:
        #postoptions.append('arabicRemoveAdditionsPhonetic')

    for options in preoptions:
      txt = getattr(PreProcess, options)(txt)

    ### Semitic Switches
    if 'novowelshebrew' in preoptions and src == 'Hebr':
        txt = txt.replace('\u05B7', '')

    srcOld = ""

    ### retain Latin ###
    if (src != 'Latn'and src != 'Type' and src in GeneralMap.SemiticScripts) or \
        (src in GeneralMap.IndicScripts and tgt in GeneralMap.SemiticScripts and src not in GeneralMap.LatinScripts) or \
            (src in ['Hiragana', 'Katakana']):
        txt = PreProcess.retainLatin(txt)

    if src == 'Hiragana' or src == 'Katakana':
        txt = PreProcess.JapanesePreProcess(src, txt, preoptions)
        srcOld = 'Japanese'
        src = 'ISO'

    if tgt == 'Hiragana' or tgt == 'Katakana':
        txt = PostProcess.JapanesePostProcess(src, tgt, txt, nativize, postoptions)

    if src == "Oriya" and tgt == "IPA":
        txt = ConvertFix.OriyaIPAFixPre(txt)

    if src == "Itrans" and '##' in txt:
        #print('I am here tyring to do things')
        transliteration = ''
        for i, word in enumerate(txt.split("##")):
            if (i%2 == 0):
                transliteration += Convert.convertScript(word, src, tgt)
            else:
                transliteration += word
    else:
        transliteration = Convert.convertScript(txt, src, tgt)

    #print(transliteration)

    if srcOld == 'Japanese' and tgt != 'Devanagari' and 'siddhammukta' not in postoptions:
        transliteration = Convert.convertScript(transliteration, "Devanagari", "ISO")
        # print(transliteration)

    if src == tgtOld:
        tgt = tgtOld
        transliteration = Convert.convertScript(transliteration, "Devanagari", tgt)

    ## apply phonetic mapping for Arabic beforehand if not semitic source
    if (src not in GeneralMap.SemiticScripts and tgt == 'Arab' and nativize) or 'arabicRemoveAdditionsPhonetic' in postoptions:
        transliteration = getattr(PostProcess, 'arabicRemoveAdditionsPhonetic')(transliteration)

    #print(transliteration)

    if nativize:
      transliteration =  PostOptions.ApplyScriptDefaults(transliteration, src, tgt, postoptions)
      if tgt != 'Latn':
        if tgt != 'Tamil':
            transliteration = PostProcess.RemoveDiacritics(transliteration)
        else:
            transliteration = PostProcess.RemoveDiacriticsTamil(transliteration)

    #print(transliteration)

    if 'RemoveDiacritics' in postoptions:
      if tgt == 'Tamil':
        postoptions = map(lambda x: 'RemoveDiacriticsTamil' if x == 'RemoveDiacritics' else x, postoptions)

    for options in postoptions:
      transliteration = getattr(PostProcess, options)(transliteration)

    if src == "Tamil" and tgt == "IPA":
        r = requests.get("http://anunaadam.appspot.com/api?text=" + txt + "&method=2")
        r.encoding = r.apparent_encoding
        transliteration = r.text

    if src == "Oriya" and tgt == "IPA":
        transliteration = ConvertFix.OriyaIPAFix(transliteration)

    '''if src == "Itrans" and '##' in txt:
        textConv = {**dict(list(json.loads(transliteration.replace("\n", "\\n")))), ** dict(textRest)}
        textConv = collections.OrderedDict(sorted(textConv.items()))
        transliteration = "".join(textConv.values())'''

    ### return Latin ###
    transliteration = PreProcess.retainLatin(transliteration, reverse=True)

    ## DefaultPostProcess ##

    transliteration = PostProcess.defaultPost(src, tgt, transliteration, nativize, preoptions, postoptions)

    return transliteration

def process(src, tgt, txt, nativize = True, post_options = [], pre_options = [], param="default"):
    if param == "default":
        return process_default(src, tgt, txt, nativize, post_options, pre_options)

    if param == "script_code":
        return process_script_tag(src, tgt, txt, nativize, post_options, pre_options)

    if param == "lang_code":
        return process_lang_tag(src, tgt, txt, nativize, post_options, pre_options)

    if param == "lang_name":
        return process_lang_name(src, tgt, txt, nativize, post_options, pre_options)

import functools

@functools.lru_cache(maxsize=None)
def _load_data(file_path):
    import os
    dir_path = os.path.dirname(os.path.realpath(__file__))
    with open(dir_path + file_path, 'r', encoding='utf8') as stream:
        data_loaded = yaml.safe_load(stream)
    return data_loaded

def convert_default(src, tgt, txt, nativize = True, post_options = [], pre_options = []):
    data_loaded = _load_data("/yaml/aksharamukha-scripts.yaml")

    scriptList = GeneralMap.IndicScripts + GeneralMap.LatinScripts + GeneralMap.SemiticScripts

    preOptionList = list(map(lambda x: x[0], getmembers(PreProcess, isfunction)))
    preOptionListLower = list(map(lambda x: x.lower(), preOptionList))

    postOptionList = list(map(lambda x: x[0], getmembers(PostProcess, isfunction)))
    postOptionListLower = list(map(lambda x: x.lower(), postOptionList))

    post_options = [option_id for option in post_options for option_id in postOptionList if option.lower() == option_id.lower()]
    pre_options = [option_id for option in pre_options for option_id in preOptionList if option.lower() == option_id.lower()]

    # font hack warning
    font_hack_warning = tgt + ' uses an hacked font to display the script. In the absence of this font, you text may appear different. \n See: https://www.aksharamukha.com/describe/' + tgt + ' for the font used'

    if tgt in data_loaded and 'font_hack' in data_loaded[tgt]:
        warnings.warn(font_hack_warning)

    if src not in scriptList:
        script_not_found = 'Source script: ' + src + ' not found in the list of scripts supported. The text will not be transliterated.'
        warnings.warn(script_not_found)

    if tgt not in scriptList:
        script_not_found = 'Target script: ' + tgt + ' not found in the list of scripts supported. The text will not be transliterated.'
        warnings.warn(script_not_found)

    if (tgt == 'RomanLoC' and src not in GeneralMap.LoCScripts):
        script_not_found = 'The LoC romanization of ' + src + ' is not yet supported. The output text will be rendered using ISO 233 if Semitic else ISO 15919. See: https://www.aksharamukha.com/loc'
        warnings.warn(script_not_found)

    if (src == 'RomanLoC' and tgt not in GeneralMap.LoCScripts):
        script_not_found = 'The LoC romanization of ' + tgt + ' is not yet supported. The input text will be treated as if it was ISO 233 if Semitic else ISO 15919.. See: https://www.aksharamukha.com/loc'
        warnings.warn(script_not_found)

    return convert(src, tgt, txt, nativize, pre_options, post_options)

def process_default(src, tgt, txt, nativize, post_options, pre_options):
    scriptList = GeneralMap.IndicScripts + GeneralMap.LatinScripts
    scriptListLower = list(map(lambda x: x.lower(), scriptList))

    if src == 'autodetect':
        src = auto_detect(txt)
        #print('The autodetection is ' + src)
        pre_options = detect_preoptions(txt, src)
    elif src.lower() in scriptListLower:
        src = [script_id for script_id in scriptList if src.lower() == script_id.lower()][0]

    if tgt.lower() in scriptListLower:
        tgt = [script_id for script_id in scriptList if tgt.lower() == script_id.lower()][0]

    return convert_default(src, tgt, txt, nativize, post_options, pre_options)


def process_script_tag(src_tag, tgt_tag, txt, nativize, post_options, pre_options):
    # Read YAML file
    data_loaded = _load_data("/yaml/aksharamukha-scripts.yaml")

    data_loaded_wiki = _load_data("/yaml/wikitra2-data.yaml")

    src = []
    tgt = []

    if src_tag == 'Syrc':
        src_tag = 'Syre'
        warnings.warn("Please specify the variety of Syriac script for the source: Estrangelo (Syre), Eastern (Syrn) or Wester (Syrj). Defaulting to Syre")
    if tgt_tag == 'Syrc':
        tgt_tag = 'Syre'
        warnings.warn("Please specify the variety of Syriac script for the target: Estrangelo (Syre), Eastern (Syrn) or Wester (Syrj). Defaulting to Syre")

    # loop through yaml to find match
    for scrpt in data_loaded.keys():
        scrpt_tag = data_loaded[scrpt]['script']

        # get the popuation of each language of the script
        if 'lang' in data_loaded[scrpt].keys():
            lang_tag = data_loaded[scrpt]['lang'].split(',')[0]
            lang = list(map(lambda x: x.lower(), data_loaded[scrpt]['lang'].split(',')))
        else:
            population = 0
            lang = ''

        #print(scrpt_tag, lang_tag)
        if scrpt_tag in data_loaded_wiki.keys() and lang_tag in data_loaded_wiki[scrpt_tag].keys():
            population = data_loaded_wiki[scrpt_tag][lang_tag]['population']
        else:
            population = 0

        # find the match in the file
        if '-' not in src_tag and data_loaded[scrpt]['script'].lower() == src_tag.lower():
            src.append((population, scrpt))

        if '-' not in tgt_tag and data_loaded[scrpt]['script'].lower() == tgt_tag.lower():
            tgt.append((population, scrpt))

        #print(population)
        # if hypthenated find the exact match
        if '-' in tgt_tag:
            lang_part = tgt_tag.split('-')[0].lower()
            script_part = tgt_tag.split('-')[1].lower()

            if scrpt_tag.lower() == script_part.lower() and 'lang' in data_loaded[scrpt].keys() and lang_part.lower() in lang:
                tgt.append((0, scrpt))

        if '-' in src_tag:
            lang_part = src_tag.split('-')[0].lower()
            script_part = src_tag.split('-')[1].lower()

            if scrpt_tag.lower() == script_part.lower() and 'lang' in data_loaded[scrpt].keys() and lang_part.lower() in lang:
                src.append((0, scrpt))

    if src_tag.lower() in ['latn', 'en', 'eng']:
        warn = "Latin has multiple transcription schemes. 'ISO 15919' has been selected by default"
        warn += "\n Please use a transcription format e.g. la-IAST or la-HK to select a particular scheme"
        warnings.warn(warn)

        src = [(0, 'ISO')]

    if tgt_tag.lower() in ['latn', 'en', 'eng']:
        warn = "Latin has multiple transcription schemes. 'ISO 15919' has been selected by default"
        warn += "\n Please use a transcription format e.g. la-IAST or la-HK to select a particular scheme"
        warnings.warn(warn)

        tgt = [(0, 'ISO')]

    # if latin ignore la and get the following part
    if '-' in src_tag and src_tag.split('-')[0].lower() in ['latn', 'en', 'eng']:
        src = [(0, src_tag.split('-')[1])]

    if '-' in tgt_tag and tgt_tag.split('-')[0].lower() in ['latn', 'en', 'eng']:
        tgt = [(0, tgt_tag.split('-')[1])]

    if src_tag == 'autodetect':
        src = [(0, auto_detect(txt))]
        pre_options = detect_preoptions(txt, src)

    if len(src) > 0:
        src_pop = sorted(src, reverse=True)[0][1]
    elif src_tag in GeneralMap.SemiticScripts:
            src_pop = src_tag
    else:
        raise Exception('Source script code: ' + src_tag + ' not found')

    if len(tgt) > 0:
        tgt_pop = sorted(tgt, reverse=True)[0][1]
    elif tgt_tag in GeneralMap.SemiticScripts:
        tgt_pop = tgt_tag
    else:
        raise Exception('Target script code: ' + tgt_tag + ' not found')

    if len(src) > 1:
        warn = "Multiple orthographies, " + ','.join(map(lambda x: x[1], src)) + ", are associated with the input script. The most popular '" + src_pop + "' has been selected"
        warn += "\n Please use the format lang_code-script_code e.g. ur-Arab or pa-Arab to select a particular orthography"

        warnings.warn(warn)
    if len(tgt) > 1:
        warn = "Multiple orthographies: " + ', '.join(map(lambda x: x[1], tgt)) + " are associated with the target script. The most popular '" + tgt_pop + "' has been selected"
        warn += "\n Please use the format lang_code-script_code e.g. ur-Arab or pa-Arab to select a particular orthography"
        warnings.warn(warn)

    return process_default(src_pop, tgt_pop, txt, nativize, post_options, pre_options)

def process_lang_tag(src_tag, tgt_tag, txt, nativize, post_options, pre_options):
    # Read YAML file
    data_loaded = _load_data("/yaml/aksharamukha-scripts.yaml")

    data_loaded_wiki = _load_data("/yaml/wikitra2-data.yaml")

    src = []
    tgt = []

    # iterate through the yaml file
    for scrpt in data_loaded.keys():
        if 'lang' in data_loaded[scrpt].keys():
            lang = list(map(lambda x: x.lower(), data_loaded[scrpt]['lang'].split(',')))
        else:
            lang = ''

        scrpt_tag = data_loaded[scrpt]['script']

        # try to get the popular script from wiktionary file
        if scrpt_tag in data_loaded_wiki.keys():
            script_count = len(data_loaded_wiki[scrpt_tag])
        else:
            script_count = 1

        # trying to find the match in the yaml file
        if src_tag.lower() in lang:
            src.append((script_count, scrpt))

        if tgt_tag.lower() in lang:
            tgt.append((script_count, scrpt))

        if '-' in tgt_tag:
            lang_part = tgt_tag.split('-')[0].lower()
            script_part = tgt_tag.split('-')[1].lower()

            if scrpt_tag.lower() == script_part and lang_part in lang:
                tgt.append((0, scrpt))

        if '-' in src_tag:
            lang_part = src_tag.split('-')[0].lower()
            script_part = src_tag.split('-')[1].lower()

            if scrpt_tag.lower() == script_part and lang_part in lang:
                src.append((0, scrpt))

    if src_tag.lower() in ['sa','san', 'pi', 'pli']:
        warn = "The input language: " + src_tag + " is script-agnostic and can be written in multiple scripts. The most popular 'Devanagari' has been selected"
        warn += "\n Please use the format lang_code-script_code e.g. sa-Gran or sa-Sidd to select a particular script"
        warnings.warn(warn)

        src = [(0, 'Devanagari')]

    if tgt_tag.lower() in ['sa','san', 'pi', 'pli']:
        warn = "The ouptput language: " + tgt_tag + " is script-agnostic and can be written in multiple scripts. The most popular 'Devanagari' has been selected"
        warn += "\n Please use the format lang_code-script_code e.g. sa-Gran or sa-Sidd to select a particular script"
        warnings.warn(warn)

        tgt = [(0, 'Devanagari')]

    # if sa-deva ignore sanskrita and only get deva
    if '-' in src_tag and src_tag.split('-')[0].lower() in ['sa','san', 'pi', 'pli']:
        for scrpt in data_loaded.keys():
            scrpt_tag = data_loaded[scrpt]['script']

            if scrpt_tag.lower() == src_tag.split('-')[1].lower():
                src = [(0, scrpt)]

    if '-' in tgt_tag and tgt_tag.split('-')[0].lower() in ['sa','san', 'pi', 'pli']:
        for scrpt in data_loaded.keys():
            scrpt_tag = data_loaded[scrpt]['script']

            if scrpt_tag.lower() == tgt_tag.split('-')[1].lower():
                tgt = [(0, scrpt)]

    if src_tag.lower() in ['la', 'en', 'eng']:
        warn = "Latin has multiple transcription schemes. 'ISO 15919' has been selected by default"
        warn += "\n Please use a transcription format e.g. Latn-IAST or Latn-HK to select a particular scheme"
        warnings.warn(warn)

        src = [(0, 'ISO')]

    if tgt_tag.lower() in ['la', 'en', 'eng']:
        warn = "Latin has multiple transcription schemes. 'ISO 15919' has been selected by default"
        warn += "\n Please use a transcription format e.g. Latn-IAST or Latn-HK to select a particular scheme"
        warnings.warn(warn)

        tgt = [(0, 'ISO')]

    # if latin ignore la and get the following part
    if '-' in src_tag and src_tag.split('-')[0].lower() in ['la', 'en', 'eng']:
        src = [(0, src_tag.split('-')[1])]

    if '-' in tgt_tag and tgt_tag.split('-')[0].lower() in ['la', 'en', 'eng']:
        tgt = [(0, tgt_tag.split('-')[1])]

    if src_tag == 'autodetect':
        src = [(0,auto_detect(txt))]
        pre_options = detect_preoptions(txt, src)

    if len(src) > 0:
        src_pop = sorted(src, reverse=True)[0][1]
    else:
        raise Exception('Source language code: ' + src_tag + ' not found')

    if len(tgt) > 0:
        tgt_pop = sorted(tgt, reverse=True)[0][1]
    else:
        raise Exception('Target language code: ' + tgt_tag + ' not found')

    if len(src) > 1:
        warn = "Multiple scripts " + ','.join(map(lambda x: x[1], src)) + " associated with the input language. The most popular '" + src_pop + "' has been selected"
        warn += "\n Please use the format lang_code-script_code e.g. pa-Guru or pa-Arab to select a particular script"
        warnings.warn(warn)
    if len(tgt) > 1:
        warn = "Multiple scripts " + ','.join(map(lambda x: x[1], tgt)) + " associated with the target language. The most popular '" + tgt_pop + "' has been selected"
        warn += "\n Please use the format lang_code-script_code e.g. pa-Guru or pa-Arab to select a particular script"
        warnings.warn(warn)

    return process_default(src_pop, tgt_pop, txt, nativize, post_options, pre_options)

def process_lang_name(src_name, tgt_name, txt, nativize, post_options, pre_options):
    if src_name == 'autodetect':
        src = auto_detect(txt)
        pre_options = detect_preoptions(txt, src)
    else:
        src = str(langcodes.find(src_name))

    tgt = str(langcodes.find(tgt_name))

    return process_lang_tag(src, tgt, txt, nativize, post_options, pre_options)

@functools.lru_cache(maxsize=None)
def get_semitic_json():
    from pathlib import Path
    cwd = Path(Path(__file__).parent)
    with open(Path(cwd, "json/gimeltra_data.json"), "r", encoding="utf-8") as f:
        data = json.load(f)

    return data

@functools.lru_cache(maxsize=None)
def getOptions(script):
    #json.load('')
    return

## add the new libraries to requiesments.txt in both folders